<?php

namespace Services;

use Exception\CustomException;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Model\Geral\Entities\Codacesslogin;
use Util\FactoryInstance;
use Util\InstanceProvider;
use Util\RepositoryInstance;
use Util\Utilities;

class TwoFactor
{

    const BASE_SUBJECT_EMAIL = 'Chave de acesso';
    const BASE_SUBJECT_EMAIL_ES = 'Clave de acceso';
    const BASE_SUBJECT_EMAIL_EN = 'Access key';
    const TYPE_LOGIN_INIT = 'I';
    const IDUTCTRLFAC_ATIVE = 'S';
    const TYPE_LOGIN_ORGANIZATION= 'S';


    public function execTwoFactors($userData, $defaultOrgId, $style, $language, $accessKey, $typeLogin)
    {
        $lifeTime = null;
        $email = null;
        if (!empty($userData['USER']["EMAIL"])) {
            $email = $userData['USER']["EMAIL"];
            $cdoperador = $userData['USER']["ID"];
            $paramSenha = RepositoryInstance::getParamgeral()->getParamSenhaByNrOrg($defaultOrgId);
            if (!empty($paramSenha) && $paramSenha[0]['IDUTCTRLFAC'] === self::IDUTCTRLFAC_ATIVE) {
                $entityManager = InstanceProvider::getLibConnectionCustom()->updateVpdConnection($defaultOrgId);
                $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
                if($typeLogin === self::TYPE_LOGIN_INIT || $typeLogin === self::TYPE_LOGIN_ORGANIZATION) {
                    FactoryInstance::getCodacesslogin()->deleteAllKeyOperador($entityManager, $cdoperador,$defaultOrgId);
                }
                if($accessKey) {
                    $this->validateCodeLogin($entityManager, $accessKey, $defaultOrgId, $cdoperador,$paramSenha[0]['NRTENAUTCHA']);
                } else {
                    $lifeTime = $paramSenha[0]['NRTEPCODAUT'];
                    $code = Utilities::getRandomAlfaNumerico();
                    $this->sendEmail($code, $email, $cdoperador, $style, $language);
                    $this->setOperadorCodeLogin($code,$cdoperador, $defaultOrgId, $lifeTime,$entityManager);
                }
            }
        }
        return array("LIFE_TIME_CODE" => $lifeTime, "EMAIL" => $email);
    }

    public function validateCodeLogin(\Doctrine\ORM\EntityManager $entityManager, $accessKey, $defaultOrgId, $cdoperador, $nrtenautcha){
        $error = null;
        /** @var  Codacesslogin $codacesslogin */
        $codacesslogin = RepositoryInstance::getCodacesslogin($entityManager)->findOneBy(
            array(
                'nrorg' => $defaultOrgId,
                'cdacesslogin' => strtoupper($accessKey),
                'cdoperador' => $cdoperador
            )
        );
        $now = new \DateTime();
        if (empty($codacesslogin)) {
            $error = 'IMVALID_KEY';
        } else {
            $nrtentativas = $codacesslogin->getNrtentativas()+1;

            if ($nrtentativas >= intval($nrtenautcha)) {
                $error = 'ATTEMPTS_KEY';
            } else {
                $dateEnd = $codacesslogin->getDtvalidade();
                if ($now > $dateEnd) {
                    $error = 'EXPIRED_KEY';
                }
            }
        }



        if (empty($error)) {
            FactoryInstance::getCodacesslogin()->deleteAllKeyOperadorFilterKey($entityManager, $cdoperador,$defaultOrgId,$accessKey);
            $codacesslogin->setStatus('S');
            $now->modify('+ 10 minutes');
            $codacesslogin->setDtvalidade($now);
            $entityManager->persist($codacesslogin);
            $entityManager->flush();
            return true;
        } else {
            /** @var  Codacesslogin[] $codacessloginArray */
            $codacessloginArray = RepositoryInstance::getCodacesslogin($entityManager)->findBy(
                array(
                    'nrorg' => $defaultOrgId,
                    'cdoperador' => $cdoperador
                )
            );
            if (!empty($codacessloginArray)) {
                foreach ($codacessloginArray as $item) {
                    $nrtentativas = $item->getNrtentativas() + 1;
                    if ($nrtentativas >= intval($nrtenautcha)) {
                        $error = 'ATTEMPTS_KEY';
                    }
                    $item->setNrtentativas($nrtentativas);
                    $item->setDtultatu(new \DateTime());
                    $entityManager->persist($item);
                }
                $entityManager->flush();
            }
            if ($error == 'ATTEMPTS_KEY') {
                \Exception\Login::attemptsKey();
            } else if ($error == 'EXPIRED_KEY') {
                \Exception\Login::expiredKey();
            } else if ($error == 'IMVALID_KEY') {
                \Exception\Login::invalidKey();
            }
        }

        return true;
    }

    public function setOperadorCodeLogin($code,$cdoperador, $defaultOrgId,$lifeTime,\Doctrine\ORM\EntityManager $entityManager) {
        /** @var \Factory\Codacesslogin $factoryCodacesslogin */
        $factoryCodacesslogin = FactoryInstance::getCodacesslogin();
        $newCode = new NewCode($entityManager->getConnection(), 'NOVO_CODIGO_SC');
        $nrcodacesslogin = $newCode->getCodeByTableName("CODACESSLOGIN", $defaultOrgId, 1, 10, true);
        $codacesslogin = $factoryCodacesslogin->create(null, $defaultOrgId, $cdoperador, $nrcodacesslogin);
        $dtvalidade = new \DateTime();
        $dtvalidade->modify('+ ' . $lifeTime . ' minutes');
        $codacesslogin = $factoryCodacesslogin->edit($codacesslogin, $defaultOrgId, $cdoperador, 0, $code, $dtvalidade, 'P', 'S');
        $entityManager->persist($codacesslogin);
        $entityManager->flush();
        $factoryCodacesslogin->updateNumberAttempts($entityManager, $cdoperador,$defaultOrgId, 0);
    }

    /**
     * @param $style
     * @return string
     */
    private function getSnedCodeTemplatePath($style, $language = null) {
        if (empty($language)) {
            $language = 'pt_br';
        }
        $baseDir = __DIR__.'/../Util/EmailTemplates/';
        $languageDir = $baseDir . $language . '/';
        $customFile = $languageDir . 'send_code_' . $style . '.html';
        $defaultFile = $languageDir . 'send_code.html';
        return file_exists($customFile) ? $customFile : $defaultFile;
    }

    /**
     * @param $cdOperator
     * @param $email
     * @param $password
     * @param $style
     * @return mixed|string
     */
    private function getReplacedTemplate($code, $email, $cdoperador, $style, $language) {
        $replacedContents = utf8_decode(file_get_contents($this->getSnedCodeTemplatePath($style, $language)));
        $replacedContents = str_replace('%codigo%', $cdoperador, $replacedContents);
        $replacedContents = str_replace('%key%', $code, $replacedContents);
        $replacedContents = str_replace('%email%', $email, $replacedContents);
        return $replacedContents;
    }

    /**
     * @param $cdOperator
     * @param $email
     * @param $password
     * @param $style
     */
    private function sendEmail($code, $email, $cdoperador, $style, $language)
    {
        $body     = utf8_encode($this->getReplacedTemplate($code, $email, $cdoperador, $style, $language));
        $smtpMail = InstanceProvider::getSmtpMail();
        $subject = self::BASE_SUBJECT_EMAIL;
        if(strpos($language, 'es') !== false) {
            $subject = self::BASE_SUBJECT_EMAIL_ES;
        } else if(strpos($language, 'en') !== false) {
            $subject = self::BASE_SUBJECT_EMAIL_EN;
        }
        $smtpMail->setSubject($subject);
        $smtpMail->setIsHTML(true);
        $smtpMail->sendMail($body, $email);
    }

}
