module.exports = function(grunt){
    var backendModules = ['../../teknisa/loginSetup', '../../teknisa/ger'];
    var directories = ['util', 'services', 'controllers'];
    var subModules = [];
    var libFiles = [
        'zh-grid-navigation/dist/GridNavigationLibrary.js',
        'zh-interval-component/dist/zh-interval-component.js',
        'zh-cache-bypass/src/CacheBypass.js',

        'zh-grid-configuration/dist/zh-grid-configuration.js',
        'zh-preferences/dist/preferences.js',
        'zh-filter-preferences/dist/filter-preferences.js',
        'zh-layout-preferences/dist/layout-preferences.js',
        'zh-view-preferences/dist/view-preferences.js',
        'zh-select-autocomplete/dist/zh-select-autocomplete.js',
        'zhuaclfrontend/assets/dist/main.js',
        'zhulog/dist/main.js',
        'tek-libraries/dist/main.js'
    ];

    grunt.initConfig({
        // grunt-contrib-concat
        concat: {
            js: {
                options: {
                    process: function (src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: getJSPaths(),
                dest: './assets/js/app.js'
            },
            buildLibs: {
                options: {
                    process: function (src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: getLibPaths(),
                dest: './assets/js/libs.js'
            }
        },

        // grunt-contrib-jshint
        jshint: {
            all: ['Gruntfile.js', './assets/js/controllers/**/*.js', './assets/js/services/**/*.js', './assets/js/util/**/*.js']
        },

        // grunt-contrib-uglify
        uglify: {
            options: {
                mangle: false,
                compress: false,
                report: 'min',
                banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
            },
            dist: {
                files: {
                    './assets/js/app.min.js': getJSPaths(),
                    './assets/js/libs.min.js': getLibPaths()
                }
            }
        },

        // grunt-contrib-cssmin
        cssmin: {
            dist: {
                files: {
                    'bower_components/zeedhi-frontend/assets/css/libs/bootstrap.min.css': 'bower_components/zeedhi-frontend/assets/css/libs/bootstrap.css',
                    'bower_components/zeedhi-frontend/assets/fonts/font-zeedhi/stylesheet.min.css': 'bower_components/zeedhi-frontend/assets/fonts/font-zeedhi/stylesheet.css',
                                        
                    'bower_components/zeedhi-frontend/assets/css/styles/styles-bigdinner.min.css': 'bower_components/zeedhi-frontend/assets/css/styles/styles-bigdinner.css',
                    'bower_components/zeedhi-frontend/assets/css/styles/styles-inflightor.min.css': 'bower_components/zeedhi-frontend/assets/css/styles/styles-inflightor.css',
                    'bower_components/zeedhi-frontend/assets/css/styles/styles-odhen.min.css': 'bower_components/zeedhi-frontend/assets/css/styles/styles-odhen.css',
                    'bower_components/zeedhi-frontend/assets/css/styles/styles-pebbian.min.css': 'bower_components/zeedhi-frontend/assets/css/styles/styles-pebbian.css',
                    'bower_components/zeedhi-frontend/assets/css/styles/styles-schoolmeals.min.css': 'bower_components/zeedhi-frontend/assets/css/styles/styles-schoolmeals.css',
                    'bower_components/zeedhi-frontend/assets/css/styles/styles-tecdiet.min.css': 'bower_components/zeedhi-frontend/assets/css/styles/styles-tecdiet.css',
                    'bower_components/zeedhi-frontend/assets/css/styles/styles-tecfood.min.css': 'bower_components/zeedhi-frontend/assets/css/styles/styles-tecfood.css',
                    'bower_components/zeedhi-frontend/assets/css/styles/styles-zeedhi.min.css': 'bower_components/zeedhi-frontend/assets/css/styles/styles-zeedhi.css',
                    
                    'bower_components/zh-select-autocomplete/dist/style/colors/styles-inflightor.min.css': 'bower_components/zh-select-autocomplete/dist/style/colors/styles-inflightor.css',
                    'bower_components/zh-grid-navigation/dist/style.min.css': 'bower_components/zh-grid-navigation/dist/style.css',
                    'bower_components/zh-preferences/dist/style/colors/styles-inflightor.min.css': 'bower_components/zh-preferences/dist/style/colors/styles-inflightor.css',
                    'assets/css/custom.min.css': 'assets/css/custom.css'
                }
            }
        },

        // grunt-contrib-watch
        watch: {
            pivotal: {
                files: getJSPaths().concat(["GruntFile.js"]),
                tasks: ['jshint', 'concat:js']
            }
        },

        // grunt-zh-id-generator
        zhIdGenerator: {
            options: {
                jsonPath: getJSONPaths(),
                menuPath: ["./assets/json/menu.json"]
            }
        },

        // grunt-zhuacl-structure
        buildAclStruct : {
            options: {
                menuConfigFile: "./menuConfig.json"
            }
        },

        buildEnvironment: {
            development: {
                configFile: "./config/devEnvironment.json"
            },
            production: {
                configFile: "./config/prodEnvironment.json"
            },
            test: {
                configFile: "./config/testEnvironment.json"
            },
            srcFolders: ["../backend/config/", "./assets/js/util", "./assets/json", "./config", "./", "./assets", "./assets/js"]
        },

        zhWords: {
            options: {
                serverUrl: "https://translate.teknisa.com/backend/index.php",
                wordsPath: "./assets/json/words/",
                organization: "20",
                project: "4807"
            }
        }
    });

    function getJSONPaths() {
        var paths = ["./assets/json"];
        subModules.forEach(function (subModule) {
            paths.push("./assets/modules/" + subModule + "/assets/json");
        });
        return paths;
    }

    function getJSPaths() {
        var paths = [];
        paths.push('./assets/js/config.js');
        directories.forEach(function (directory) {
            paths.push('./assets/js/' + directory + '/**/*.js');
            subModules.forEach(function (subModule) {
                paths.push('./assets/modules/' + subModule + '/assets/js/' + directory + '/**/*.js');
            });
        });
        return paths;
    }

    function getLibPaths() {
        var paths = [];
        libFiles.forEach(function(fileName) {
            paths.push('bower_components/' + fileName);
        });
        return paths;
    }

    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-uglify');
    grunt.loadNpmTasks('grunt-contrib-cssmin');
    grunt.loadNpmTasks('grunt-contrib-concat');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks('grunt-zh-id-generator');
    grunt.loadNpmTasks('grunt-zhuacl-structure');
    grunt.loadNpmTasks('grunt-zh-environment-builder');
    grunt.loadNpmTasks('grunt-zh-words');

    grunt.registerTask('default', ['jshint', 'buildEnvironment', 'concat', 'uglify', 'cssmin', 'buildAclStruct', 'zhWords']);
    grunt.registerTask('buildLibs', ['concat:buildLibs']);
    grunt.registerTask('commit', ['jshint', 'uglify', 'concat']);
    grunt.registerTask('basic', ['uglify', 'cssmin', 'concat:js', 'zhIdGenerator']);
};